<?php

namespace App\Exports;

use App\Models\DivisionModel;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Illuminate\Support\Facades\Session;
use Maatwebsite\Excel\Events\AfterSheet;
use Illuminate\Support\Facades\DB;
use App\Models\VehicleTrakingReportModel;
use App\Models\VehicleModel;

class CheckpointArrivalExport implements FromArray,WithHeadings,ShouldAutoSize,WithEvents
{
    protected $data;
    protected $headings;
    protected $columnWidth = [];
    protected $rowHeight = [];
    protected $mergeCells = [];
    protected $font = [];
    protected $fontSize = [];
    protected $bold = [];
    protected $background = []; 
    protected $vertical = []; 
    protected $sheetName;
    protected $borders = [];

    /**
    * @return \Illuminate\Support\Collection
    */
    public function array(): array
    {
        $aData = array();

        $aData['division_id']     = session::has('division_id') ? session::get('division_id') : 0;
        $aData['ward_id']         = session::has('ward_id') ? session::get('ward_id') : 0;
        $aData['vehicle_id']      = session::has('vehicle_id') ? session::get('vehicle_id') : 0;
        $aData['vehicle_type_id'] = session::has('vehicle_type_id') ? session::get('vehicle_type_id') : 0;
        $aData['date']            = session::has('date') ? session::get('date') : date('d-m-Y',strtotime('now'));

        $aData['records']      = VehicleTrakingReportModel::get_checkpoint_arrival($aData);
        if( $aData['records']){
            foreach($aData['records'] as $key=>$aValue){ 
                foreach($aValue['checkpoints'] as $val){

                    if(!empty($val->distance)){ 
                        $actual_reach_time = (!empty($val->actual_reach_time)) ?  $val->actual_reach_time : 'Not Arrived'; 
                        $time_difference = (!empty($val->time_difference)) ? $val->time_difference : '-'; 
                        $distance = (!empty($val->distance)) ? number_format($val->distance, 2).' Meter' : "-";
                    }else{
                        $actual_reach_time = "Not Arrived";
                        $time_difference = " ";
                        $distance = " ";
                    }
                   
                    $aData['excel'][] = array(
                        'route_name' => $val->route_name,
                        'vehicle_name' => (!empty($aValue['vehicle_name'])) ? $aValue['vehicle_name'] : "-",
                        'expected_time' => $val->expected_time ? $val->expected_time : "-",
                        'actual_reach_time' => $actual_reach_time,
                        'time_difference' => $time_difference,
                        'distance' => $distance,
                        'location' => $val->checkpoint_location ? $val->checkpoint_location : "-"
                    ); 
                }
            }
        }else{
            $aData['excel'][] = array(
            );     
        }
        return $aData['excel'];
    }

    public function headings(): array
    {   
        $aData['division_id']     = Session::has('division_id') ? Session::get('division_id') : 0;
        $aData['ward_id']         = Session::has('ward_id') ? Session::get('ward_id') : 0;
        $aData['vehicle_id']      = Session::has('vehicle_id') ? Session::get('vehicle_id') : 0;
        $aData['vehicle_type_id'] = Session::has('vehicle_type_id') ? Session::get('vehicle_type_id') : 0;
        $aData['date']            = Session::has('date') ? Session::get('date') : date('d-m-Y',strtotime('now'));

        $division_name = "-";
        if($aData['division_id'] != 0){
            $div = DivisionModel::where('id', $aData['division_id'])->get('division_name');
            foreach($div as $val){
                $division_name = $val->division_name;
            }
        }
        $ward_name = "-";
        if($aData['ward_id'] != 0){
            $ward_id = DB::table('admin_wards')->where('id',  $aData['ward_id'])->get('ward_name'); 
            foreach($ward_id as $val){
                $ward_name = $val->ward_name;
            }
        }

        $vehicle_number = "-";
        if($aData['vehicle_id'] != 0){
            $vehicle= VehicleModel::where('id', $aData['vehicle_id'])->get('vehicle_number');
            foreach($vehicle as $val){
                $vehicle_number = $val->vehicle_number;
            }
        }

        $vehicle_types = "-";
        if($aData['vehicle_type_id'] != 0){
            $vehicle = DB::table('vehicle_types')->where('id',  $aData['vehicle_type_id'])->get('type'); 
            foreach($vehicle as $val){
                    $vehicle_types = $val->type;
            }
        }
               
        return [
                ['Report Name: Checkpoint Arrival Report.'],
                ['Division Name : '. $division_name ],
                ['Ward Name : '.$ward_name],
                ['Vehicle Number : '.$vehicle_number ],
                ['Vehicle Type : '.$vehicle_types ],
                ['Date : '. $aData['date']],
                [],
                [ 
                    'Route Name ',
                    'Vehicle Number ',
                    'Expected Time ',
                    'Actual Time ',                   
                    'Time Difference ',                   
                    'Distance ',                   
                    'Location ',                   
                ]
        ];
    }

      /**
     * @return string
     */
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function(AfterSheet $event) {
                $event->sheet->getDelegate()->getStyle('A1:Z1265')->getAlignment()->setHorizontal('left');
                $event->sheet->mergeCells('A1:D1');
                $event->sheet->mergeCells('A2:D2');
                $event->sheet->mergeCells('A3:D3');
                $event->sheet->mergeCells('A4:D4');
                $event->sheet->mergeCells('A5:D5');
                $event->sheet->mergeCells('A6:D6');
                $event->sheet->mergeCells('A7:Z7');
                
                $aData = array();
                $aData['division_id']     = Session::has('division_id') ? Session::get('division_id') : 0;
                $aData['ward_id']         = Session::has('ward_id') ? Session::get('ward_id') : 0;
                $aData['vehicle_id']      = Session::has('vehicle_id') ? Session::get('vehicle_id') : 0;
                $aData['vehicle_type_id'] = Session::has('vehicle_type_id') ? Session::get('vehicle_type_id') : 0;
                $aData['date']            = Session::has('date') ? Session::get('date') : date('d-m-Y',strtotime('now'));
                $aData['records']      = VehicleTrakingReportModel::get_checkpoint_arrival($aData);
                $counter = 0;
                $rowSpan = 8;
                $p = 9;

                foreach($aData['records'] as $key=>$aValue){ 
                    
                    if(!empty($aValue['checkpoints'])){
                        $check_point = $aValue['checkpoints'];   
                        $rowSpan += count($check_point);
                        $event->sheet->mergeCells('A'.$p.':A'.$rowSpan);
                        $event->sheet->mergeCells('B'.$p.':B'.$rowSpan);
                        $event->sheet->getDelegate()->getStyle('A'.$p.':A'.$rowSpan)->getAlignment()->setVertical('center');
                        $event->sheet->getDelegate()->getStyle('B'.$p.':B'.$rowSpan)->getAlignment()->setVertical('center');
                        $counter ++;
                        $p =  $rowSpan+1;
                    }
                }

                $event->sheet->getDelegate()->getRowDimension('1')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('2')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('3')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('4')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('5')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('6')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('8')->setRowHeight(25);
              
                $event->sheet->getStyle('A1:Z8')->applyFromArray([
                    'font'=>[
                            'bold'=>true,
                    ]
                ]);

                $highestRow = $event->sheet->getHighestRow();
                $highestColumn = $event->sheet->getHighestColumn();
                
                for ($row = 9; $row <= $highestRow; ++$row) {
                    $actualOnMin = $event->sheet->getCellByColumnAndRow(4,$row)->getValue();
                    if($actualOnMin == 'Not Arrived'){
                        $event->sheet->mergeCells('D'.$row.':F'.$row);
                        $event->sheet->getDelegate()->getStyle('D'.$row.':F'.$row)->getAlignment()->setHorizontal('center');
                    }
                }
            },
        ];
    }
  
}

