<?php

namespace App\Exports;

use App\Models\DivisionModel;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Illuminate\Support\Facades\Session;
use Maatwebsite\Excel\Events\AfterSheet;
use Illuminate\Support\Facades\DB;
use App\Models\VehicleTrakingReportModel;

class DatewiseWeightExport implements FromArray,WithHeadings,ShouldAutoSize,WithEvents
{
    protected $data;
    protected $headings;
    protected $columnWidth = [];
    protected $rowHeight = [];
    protected $mergeCells = [];
    protected $font = [];
    protected $fontSize = [];
    protected $bold = [];
    protected $background = []; 
    protected $vertical = []; 
    protected $sheetName;
    protected $borders = [];

    /**
    * @return \Illuminate\Support\Collection
    */
    public function array(): array
    {
        $aData = array();
        $aData['division_id']     = Session::has('division_id') ? Session::get('division_id') : 0;
        $aData['vehicle_type_id'] = Session::has('vehicle_type_id') ? Session::get('vehicle_type_id') : 0;
        $aData['start_date']      = Session::has('start_date') ? Session::get('start_date') : date('d-m-Y',strtotime('now'));
        $aData['end_date']        = Session::has('end_date') ? Session::get('end_date') : date('d-m-Y',strtotime('now'));
       
        $aData['divisionWeightData'] = array();
        if(!empty($aData['division_id'])){
			$aData['divisionWeightData'] = VehicleTrakingReportModel::get_date_wise_weight_mngmt($aData);
        }      
        $total_tons = 0;
        if( $aData['divisionWeightData']){
            foreach($aData['divisionWeightData'] as $key=>$aRec){ 
                $total_net = $aRec->total_net;
                $tons = $total_net/1000;
                $total_tons += $tons;
                $aData['excel'][] = array(
                    'current_day' => date('d-m-Y',strtotime($aRec->current_day)),
                    'total_loaded' => $aRec->total_loaded,
                    'total_unloaded' => $aRec->total_unloaded,
                    'total_net' => $aRec->total_net,
                    'tons' => $tons
                ); 
            }
            $aData['excel'][] = array(
            
                'Total' =>'Total' ,
                '',
                '',
                '',
                'total_tons' => $total_tons
            ); 
        }else{
            $aData['excel'][] = array(
            );     
        }
        return $aData['excel'];
    }

    public function headings(): array
    {   
        $aData['division_id']     = Session::has('division_id') ? Session::get('division_id') : 0;
        $aData['vehicle_type_id'] = Session::has('vehicle_type_id') ? Session::get('vehicle_type_id') : 0;
        $aData['start_date']      = Session::has('start_date') ? Session::get('start_date') : date('d-m-Y',strtotime('now'));
        $aData['end_date']        = Session::has('end_date') ? Session::get('end_date') : date('d-m-Y',strtotime('now'));

        $division_name = "-";
        if($aData['division_id'] != 0){
            $div = DivisionModel::where('id', $aData['division_id'])->get('division_name');
                foreach($div as $val){
                    $division_name = $val->division_name;
                }
        }

        $vehicle_types = "-";
        if($aData['vehicle_type_id'] != 0){
                $vehicle = DB::table('vehicle_types')->where('id',  $aData['vehicle_type_id'])->get('type'); 
                foreach($vehicle as $val){
                        $vehicle_types = $val->type;
                }
        }
               
        return [
                ['Report Name: Date-wise Single Division Weight Report'],
                ['Division Name : '. $division_name ],
                ['Vehicle Type : '.$vehicle_types ],
                ['Start Date : '. $aData['start_date']],
                ['End Date : '. $aData['end_date']],
                [],
                [ 
                    'Date ',
                    'Loaded Weight ',
                    'Unloaded Weight ',
                    'Net Weight ',                   
                    'Tons '                 
                ]
        ];
    }

      /**
     * @return string
     */
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function(AfterSheet $event) {
                $event->sheet->getDelegate()->getStyle('A1:Z1265')->getAlignment()->setHorizontal('left');

                $event->sheet->mergeCells('A1:D1');
                $event->sheet->mergeCells('A2:D2');
                $event->sheet->mergeCells('A3:D3');
                $event->sheet->mergeCells('A4:D4');
                $event->sheet->mergeCells('A5:D5');
                $event->sheet->mergeCells('A6:Z6'); 
                $event->sheet->getDelegate()->getRowDimension('1')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('2')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('3')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('4')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('5')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('6')->setRowHeight(25);
              
                $event->sheet->getStyle('A1:Z7')->applyFromArray([
                    'font'=>[
                            'bold'=>true,
                    ]
                ]);
                $aData = array();
                $aData['division_id']     = Session::has('division_id') ? Session::get('division_id') : 0;
                $aData['vehicle_type_id'] = Session::has('vehicle_type_id') ? Session::get('vehicle_type_id') : 0;
                $aData['start_date']      = Session::has('start_date') ? Session::get('start_date') : date('d-m-Y',strtotime('now'));
                $aData['end_date']        = Session::has('end_date') ? Session::get('end_date') : date('d-m-Y',strtotime('now'));
               
                $aData['divisionWeightData'] = array();
                
                if(!empty($aData['division_id'])){
                    $aData['divisionWeightData'] = VehicleTrakingReportModel::get_date_wise_weight_mngmt($aData);
                }      
                $i = 8;
                if( $aData['divisionWeightData']){
                    $data = count($aData['divisionWeightData']);
                    $i += $data;
                    $event->sheet->getStyle('A'.$i.':Z'.$i)->applyFromArray([
                        'font'=>[
                                'bold'=>true,
                        ]
                    ]);
                    $event->sheet->getDelegate()->getStyle('A'.$i.':D'.$i)->getAlignment()->setHorizontal('right');
                    $event->sheet->mergeCells('A'.$i.':D'.$i); 

                }
                  
            },
        ];
    }
  
}

