<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Illuminate\Support\Facades\Session;
use App\Models\ReportModel;
use Maatwebsite\Excel\Events\AfterSheet;
use Illuminate\Support\Facades\DB;
use DateTime;
use PhpOffice\PhpSpreadsheet\Worksheet\Drawing;

class EmployeeAttendanceExport implements FromArray,WithHeadings,ShouldAutoSize,WithEvents
{
    protected $data;
    protected $headings;
    protected $columnWidth = [];
    protected $rowHeight = [];
    protected $mergeCells = [];
    protected $font = [];
    protected $fontSize = [];
    protected $bold = [];
    protected $background = []; 
    protected $vertical = []; 
    protected $sheetName;
    protected $borders = [];

    /**
    * @return \Illuminate\Support\Collection
    */
    public function array(): array
    {
        $aData = array();
        $aData['excel'] = array();

        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 0;
        $aData['attendance_date'] = session::has('attendance_date') ? session::get('attendance_date') : date('d-m-Y');
        
        $aData['aAttendance'] = ReportModel::get_employee_attendance($aData)->get();

        foreach($aData['aAttendance'] as $val){
            $datetime1 = new DateTime(date('Y-m-d H:i:s', $val->inward_datetime));
            $datetime2 = new DateTime(date('Y-m-d H:i:s', $val->outword_datetime));
            if($val->outword_datetime){
                $difference = $datetime1->diff($datetime2);
                $difference = $difference->format('%H:%I');
            }else{
                $difference = "";
            }

            $aData['excel'][] = array(
                'Pakage_name' => $val->Pakage_name,
                'status' => $val->success == 1 ? 'Present' : 'Absent',
                'supervisor_name'=> $val->supervisor_name,
                'employee_id' => $val->emp_id,
                'name' => $val->name ,
                'login_time' => date('h:i A', $val->inward_datetime),
                'in_location' => $val->in_location ? $val->in_location : '-' ,
                '',
                'logout_time' => ($val->outword_datetime) ? date('h:i A', $val->outword_datetime) : " ",
                'out_location' => $val->out_location ? $val->out_location : "-" ,
                '',
                'hours' => $difference ? $difference : "-"
            );     
        }
        return $aData['excel'];
    }

    public function headings(): array
    {        
       $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 0;
        $aData['attendance_date'] = session::has('attendance_date') ? session::get('attendance_date') : date('d-m-Y');
        
        $result = DB::table('admin_pakages')
                ->where('id', $aData['Pakage_id'])
                ->get();
        
        foreach($result as $val){
            $Pakage_name  = $val->Pakage_name ;
        }

        if(!empty($Pakage_name)){
            $division  = $Pakage_name;
        }  
        else{
            $division  = '-';
        }

        return [
                ['Report Name: Employee Attendance Report.'],
                ['Pakage :'.$division],
                ['Date :'. $aData['attendance_date']],
                [ 
                    'Pakage Name',
                    'Status',
                    'Supervisor',
                    'Emp Code',
                    'Emp Name',
                    'Login Time ',
                    'Login Location',
                    'Login Image',
                    'Logout Time',
                    'Logout Location',
                    'Logout Image',
                    'Hours'
                ]
            ];
    }

      /**
     * @return string
     */
    public function registerEvents(): array
    {
        
        return [
            AfterSheet::class => function(AfterSheet $event) {
                $event->sheet->getDelegate()->getStyle('A1:Z1265')->getAlignment()->setHorizontal('left');
                $event->sheet->mergeCells('A1:D1');
                $event->sheet->mergeCells('A2:D2');
                $event->sheet->mergeCells('A3:D3');

                $event->sheet->getDelegate()->getRowDimension('1')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('2')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('3')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('4')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('6')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('7')->setRowHeight(25);

                $event->sheet->getStyle('A1:Z4')->applyFromArray([
                    'font'=>[
                            'bold'=>true,
                    ]
                ]);
                
                //display images
                $workSheet = $event->sheet->getDelegate();
                foreach ($this->getDrawings("a") as $drawing) {
                    $drawing->setWorksheet($workSheet);
                }

                foreach ($this->getDrawings("b") as $drawing) {
                    $drawing->setWorksheet($workSheet);
                }
            },
        ];
    } 
    public function getDrawings($a):array
    {
        $aData = array();
        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 0;
        //$aData['start_date'] = session::has('start_date') ? session::get('start_date') : date('d-m-Y',strtotime('-1 month'));
        //$aData['end_date'] = session::has('end_date') ? session::get('end_date') :date('d-m-Y');
        $aData['attendance_date'] = session::has('attendance_date') ? session::get('attendance_date') : date('d-m-Y');
        
        $aData['aAttendance'] = ReportModel::get_employee_attendance($aData)->get();
        $rowNum = 5;
        $drawings = [];
        
            foreach ($aData['aAttendance'] as $val) {
                $drawing = new Drawing();
                if($a == "a"){
                    if($val->in_image_url){
                        $drawing->setPath(public_path('attendance/' .date('Ymd', $val->attendance_date).'/'. $val->in_image_url));
                        $drawing->setCoordinates('H'.$rowNum);
                    }else{
                        $drawing->setPath(public_path('app-assets/images/portrait/small/proflie-default.png'));
                        $drawing->setCoordinates('H'.$rowNum);
                    }
                }
                
                if($a == "b"){
                    if($val->out_image_url){
                        $drawing->setPath(public_path('attendance/' .date('Ymd', $val->attendance_date).'/'. $val->out_image_url));
                        $drawing->setCoordinates('K'.$rowNum);
                    }else{
                        $drawing->setPath(public_path('app-assets/images/portrait/small/proflie-default.png'));
                        $drawing->setCoordinates('K'.$rowNum);
                    }
                }
    
                $drawing->setHeight(25);
                $drawing->setWidth(25);
                $drawings[] = $drawing;
                $rowNum++;
            }
            return $drawings;
    }

}

