<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Illuminate\Support\Facades\Session;
use App\Models\ReportModel;
use Maatwebsite\Excel\Events\AfterSheet;
use PhpOffice\PhpSpreadsheet\Worksheet\Drawing;


class EmployeeWiseExport implements FromArray,WithHeadings,ShouldAutoSize,WithEvents
{
    protected $data;
    protected $headings;
    protected $columnWidth = [];
    protected $rowHeight = [];
    protected $mergeCells = [];
    protected $font = [];
    protected $fontSize = [];
    protected $bold = [];
    protected $background = []; 
    protected $vertical = []; 
    protected $sheetName;
    protected $borders = [];

    /**
    * @return \Illuminate\Support\Collection
    */
    public function array(): array
    {
        $aData = array();
        $aData['emp_id'] = Session::has('emp_id') ? Session::get('emp_id') : "";
        $aData['start_date'] = Session::has('start_date') ? Session::get('start_date') : date('d-m-Y',strtotime('-1 month'));
        $aData['end_date'] = Session::has('end_date') ? Session::get('end_date') :date('d-m-Y');
		$aEmployeeData = ReportModel::getSingleData($aData);
        
        foreach($aEmployeeData as $key=>$data){		
			if($data->inward_datetime < $data->outword_datetime AND !empty($data->inward_datetime) AND !empty($data->outword_datetime)){	
				$hours = ReportModel::secondsToTime($data->hours);
				$aEmployeeData[$key]->hours = $hours['h'].':'.$hours['m'];
			}	
			else{
				$aEmployeeData[$key]->hours = '';
            }
		}
        
        if($aEmployeeData){
            foreach($aEmployeeData as $val){
                $aData['excel'][] = array(
                    'date' =>date("d-m-Y", $val->attendance_date),
                    'in_location' => $val->in_location,
                    'login_time' =>date("h:i A", $val->inward_datetime),
                    '',
                    'out_location' => $val->out_location ? $val->out_location :"-",
                    'logout_time' => $val->outword_datetime ? date("h:i A", $val->outword_datetime) : "-",
                    '',
                    'status' => $val->success? "Present" : "Absent",
                    'hours' => $val->hours ? $val->hours : "-"
                );     
            }
        }else{
            $aData['excel'][] = array(
            ); 
        }
        
        return $aData['excel'];
    }

    public function headings(): array
    {        
        $aData['emp_id'] = Session::has('emp_id') ? Session::get('emp_id') : "-";
        $aData['start_date'] = Session::has('start_date') ? Session::get('start_date') : date('d-m-Y',strtotime('-1 month'));
        $aData['end_date'] = Session::has('end_date') ? Session::get('end_date') :date('d-m-Y');
        $totalHours = ReportModel::getSingleHours($aData);
       
        foreach($totalHours as $val){
            $hours = ReportModel::secondsToTime($val->hours);
            $aData['totalHours'] = $hours['h'].':'.$hours['m'];
        }
        
        $aData['aEmployee'] = ReportModel::getEmployee($aData);
        if($aData['aEmployee']){
            foreach($aData['aEmployee'] as $val){
                $employee = $val->name;
                $pakage_name = $val->Pakage_name;
            }
        }else{
            $employee = "-";
            $pakage_name = "-";
        }
        return [
            ['Report Name: Employee Wise Report.'],
            ['Employee Name : '.$employee],
            ['Pakage Name: '.$pakage_name],
            ['Emp Id : '.$aData['emp_id'] ],
            ['Total Hours: '. $aData['totalHours']],
            ['Start Date : '.$aData['start_date']],
            ['End Date : '.$aData['end_date']],
            [],
            [ 
                'Date',
                'Login Location',
                'Login Time',
                'Login Image',
                'Logout Location',
                'Logout Time',
                'Logout Image',
                'Status',
                'Hours'
            ]
        ];
    }

      /**
     * @return string
     */
    public function registerEvents(): array
    {
        
        return [
            AfterSheet::class => function(AfterSheet $event) {
                $event->sheet->getDelegate()->getStyle('A1:Z1265')->getAlignment()->setHorizontal('left');
                $event->sheet->mergeCells('A1:C1');
                $event->sheet->mergeCells('A2:C2');
                $event->sheet->mergeCells('A3:C3');
                $event->sheet->mergeCells('A4:C4');
                $event->sheet->mergeCells('A5:C5');
                $event->sheet->mergeCells('A6:C6');
                $event->sheet->mergeCells('A7:C7');
                $event->sheet->mergeCells('A8:Z8');

                $event->sheet->getDelegate()->getRowDimension('1')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('2')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('3')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('4')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('5')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('6')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('7')->setRowHeight(25);

                $event->sheet->getDelegate()->getRowDimension('9')->setRowHeight(25);

                $event->sheet->getStyle('A1:Z9')->applyFromArray([
                    'font'=>[
                            'bold'=>true,
                    ]
                ]);
                //display images
                $workSheet = $event->sheet->getDelegate();
                foreach ($this->getDrawings("a") as $drawing) {
                    $drawing->setWorksheet($workSheet);
                }

                foreach ($this->getDrawings("b") as $drawing) {
                    $drawing->setWorksheet($workSheet);
                }
            },
        ];
    }
    
    public function getDrawings($a):array
    {
        $aData = array();
        $aData['emp_id'] = Session::has('emp_id') ? Session::get('emp_id') : "";
        $aData['start_date'] = Session::has('start_date') ? Session::get('start_date') : date('d-m-Y',strtotime('-1 month'));
        $aData['end_date'] = Session::has('end_date') ? Session::get('end_date') :date('d-m-Y');
		$aEmployeeData = ReportModel::getSingleData($aData);
        $rowNum = 10;
        $drawings = [];
        // dd($aEmployeeData);
        
            foreach ($aEmployeeData as $val) {
                $drawing = new Drawing();
                if($a == "a"){
                    if($val->in_image_url){
                        $drawing->setPath(public_path('attendance/' .date('Ymd', $val->attendance_date).'/'. $val->in_image_url));
                        $drawing->setCoordinates('D'.$rowNum);
                    }else{
                        $drawing->setPath(public_path('app-assets/images/portrait/small/proflie-default.png'));
                        $drawing->setCoordinates('D'.$rowNum);
                    }
                }
                
                if($a == "b"){
                    if($val->out_image_url){
                        $drawing->setPath(public_path('attendance/' .date('Ymd', $val->attendance_date).'/'. $val->out_image_url));
                        $drawing->setCoordinates('G'.$rowNum);
                    }else{
                        $drawing->setPath(public_path('app-assets/images/portrait/small/proflie-default.png'));
                        $drawing->setCoordinates('G'.$rowNum);
                    }
                }
    
                $drawing->setHeight(25);
                $drawing->setWidth(25);
                $drawings[] = $drawing;
                $rowNum++;
            }
            return $drawings;
        
    }
}

