<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Illuminate\Support\Facades\Session;
use Maatwebsite\Excel\Events\AfterSheet;
use App\Models\VehicleTrakingReportModel;

class PresentAbsentVehicleExport implements FromArray,WithHeadings,ShouldAutoSize,WithEvents
{
    protected $data;
    protected $headings;
    protected $columnWidth = [];
    protected $rowHeight = [];
    protected $mergeCells = [];
    protected $font = [];
    protected $fontSize = [];
    protected $bold = [];
    protected $background = []; 
    protected $vertical = []; 
    protected $sheetName;
    protected $borders = [];

    /**
    * @return \Illuminate\Support\Collection
    */

    function __construct($division_id, $date,$entrsType,$vehicle_type) {
        $this->division_id = $division_id;
        $this->date = $date;
        $this->entrsType = $entrsType;
        $this->vehicle_type = $vehicle_type;

        $aData = array();
        if(!empty($this->vehicle_type)){
            $aData['vehicle_type'] = $this->vehicle_type;
        }else{
            $aData['vehicle_type'] = Session::has('vehicle_type') ? Session::get('vehicle_type') : 0;
        }

        if(!empty($this->division_id)){
			$division_id =$this->division_id;
		}else{
            $division_id = Session::has('division_id') ? Session::get('division_id') : 0;
        }
       
        if(!empty($aData['vehicle_type'])){
			$aData['vehicle_type_name'] = VehicleTrakingReportModel::get_vehicle_type_name($aData['vehicle_type']);
        }
		else{
			$aData['vehicle_type_name'] = '';
        }
        $this->vehicle_type_name =$aData['vehicle_type_name'];

        $aData['date'] = $this->date;	
		$aData['division_name']	= '';
		if(!empty($division_id)){
			$aData['division_name'] = VehicleTrakingReportModel::get_division_name($division_id);	
		}
        $this->division_name = $aData['division_name'];

        $aData['records'] = array();
		if( $this->entrsType == "1"){
			$aData['title'] = 'Present';
			$aData['records'] =VehicleTrakingReportModel::get_present_entries($division_id,$aData['date'],$aData['vehicle_type']);
		}
        elseif($this->entrsType == "2"){
			$aData['title'] = 'Absent';
			$aData['records'] = VehicleTrakingReportModel::get_absent_entries($division_id,$aData['date'],$aData['vehicle_type']);
		}
        elseif($this->entrsType == "3"){
			$aData['title'] = 'Total';
			$aData['records'] = VehicleTrakingReportModel::get_total_entries($division_id,$aData['date'],$aData['vehicle_type']);
		}

        $this->data = $aData['records'];
    }

    public function array(): array
    {
        $aData['excel'] = array();
        foreach($this->data as $key=>$val){ 
            $aData['excel'][] = array(
                'route_name' => $val->route_name,
                'vehicle_name' => $val->vehicle_name
            ); 
        }
        return $aData['excel'];
    }

    public function headings(): array
    {   
        
        
        return [
            ['Report Name : Present and Absent Vehicle Report.'],
            ['Date : '. $this->date],
            ['Division Name : '. $this->division_name ],
            ['Vehicle Type : '.$this->vehicle_type_name],
            [],
            [ 
                'Route Name ',
                'Vehicle Number '
            ]
        ];
    }

      /**
     * @return string
     */
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function(AfterSheet $event) {
                $event->sheet->getDelegate()->getStyle('A1:Z1265')->getAlignment()->setHorizontal('left');
                $event->sheet->getStyle('A1:Z6')->applyFromArray([
                    'font'=>[
                            'bold'=>true,
                    ]
                ]);
            },
        ];
    }
  
}

