<?php

namespace App\Http\Controllers;

use App\Models\Employee;
use App\Models\EmployeeImage;
use Illuminate\Http\Request;
use App\Libraries\Mysecurity;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\DB;
use App\Models\AdminModel;
use App\Models\PakageModel;
use App\Models\DesignationModel;
use App\Imports\EmployeedetailsImport;
use Maatwebsite\Excel\Facades\Excel;

class EmployeeController extends Controller
{
    public function __construct()
    {
        $module_link = request()->segments(0);
        if(isset($module_link[0])){
            $module_link = $module_link[0];
            $this->mysecurity = new Mysecurity();
            $this->module_id = $this->mysecurity->get_module_id_by_name($module_link);
            $this->middleware(function ($request, $next){
                $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        
                return $next($request);
            });
        }
    }

    public function clear_search(){
        // return 'here';
         session::forget('Pakage_id');
         session::forget('employee_name');
         session::forget('emp_id');


         return redirect()->route('employee.list');
    }

    public function list(Request $request)
    {
        $aData = array();        
        $aData['search'] = '';
        if($request->form_type=='search_frm'){   
            session::put(['Pakage_id' => $request->Pakage_id]);
            session::put(['employee_name' => $request->employee_name]);
            session::put(['emp_id' => $request->emp_id]);
        }
        $aData['Pakage_id'] = session::has('Pakage_id') ? session::get('Pakage_id') : 0;
        $aData['employee_name'] = session::has('employee_name') ? session::get('employee_name') : "";
        $aData['emp_id'] = session::has('emp_id') ? session::get('emp_id') : "";
        // return  $aData['emp_id'];

         if(!empty($aData['Pakage_id']) || !empty($aData['employee_name']) || !empty($aData['emp_id'])){
             $aData['search'] = 1;
         }
        $aData['aEmployee'] = Employee::get($aData)->orderBy('id', 'DESC')->paginate(config('custom.page_limit'));
        $aData['aPakage'] = PakageModel::get_Pakages()->get();

        return view('employee.list',$aData);
    }

    public function search(Request $request){
        session(['Pakage_id' => $request->Pakage_id]);
        session(['employee_name' => $request->employee_name]);
        session(['emp_id' => $request->emp_id]);
        return redirect()->route('employee.list');  
    }

    public function add_edit(Request $request, $id=null){

        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $aData = array(); 
        if ($id == '') {
            $employee = new Employee();

            if(empty($request->designation)){
                $employee->designation = 5;
            }
        } else {
            $employee = Employee::find($id);
            // dd( $employee);
            
        }

        //echo '<pre>';print_r($aData['aEmployee']);die;

        $aData['aPakage'] = PakageModel::get_Pakages()->get();

        if($request->isMethod('post')){

            $this->validate($request,[
                'Pakage_id' => 'required',
                'emp_id' => 'required|unique:biometric_users,emp_id,'. $id .'id',
                'firstname' => 'required',
                'middlename' => 'required',
                'lastname' => 'required',
                'gender' => 'required',
                'address' => 'required',
            ]);
        
            $employee->Pakage_id = $request->Pakage_id;
            $employee->emp_id = $request->emp_id;
            $employee->firstname = $request->firstname;
            $employee->middlename = $request->middlename;
            $employee->lastname = $request->lastname;
            $employee->gender = $request->gender;

            if($request->designation){
                $employee->designation = $request->designation;
            }
            $employee->address = $request->address;
            $employee->save();       
            return redirect()->route('employee.list')->with('success', 'Employee Details Saved Successfully');     
        }else{ 
            $aData['aEmployee'] =$employee;
            $aData['blood_group'] = array(
                                        '1' => 'O+', 
                                        '2' => 'O-',
                                        '3' => 'A+',
                                        '4' => 'A-',
                                        '5' => 'B+',
                                        '6' => 'B-',
                                        '7' => 'AB+',
                                        '8' => 'AB-',
                                       
                                    );
            //Show master_designation
          
            $aData['designation'] = DesignationModel::get_designation()->get();

            return view('employee.add_edit', $aData); 
        }
    }

    /*IMAGE UPLOAD */

    public function image(Request $request, $id){
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        $aData = array(); 
        $aData['Emp_id'] = $id;
        $aData['Emp_image'] = EmployeeImage::where('user_id', $id)->first();
        // dd($aData['Emp_id']);
        return view('employee.image_upload', $aData); 
    }

    public function image_upload(Request $request, $id){
     
	    $request->validate([
            'image' => 'required|mimes:jpeg,jpg,png,gif|max:2000',
         ]);
         $request->input();

         if ($request->hasFile('image')) 
         {
             $image = $request->file('image');
             $imagefile = time().'.'.$image->getClientOriginalExtension();
             $destinationPath = public_path('/uploads/employee_images');
             $image->move($destinationPath, $imagefile);
         }

         $employee_image = EmployeeImage::updateOrCreate(
            ['user_id' =>  $id],
            ['image_url' => $imagefile, 'image_title' => $imagefile, 'updated_date' => strtotime("now")],        
            );
        
         return redirect()->route('employee.list')->with('success', 'Employee Image Saved Successfully');  
    }

    public function delete($id)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        Employee::find($id)->delete();
        return back()->with('success', 'Employee Record deleted successfully');
    }

    /*ATTENDANCE */

    public function attendance_log(Request $request, $id, $emp_id){
        $aData = array(); 

        $aData = Employee::get_attendance_log($request->date, $id, $emp_id);
       // dd($request);
        if($request->in_entry == 'Mark as IN'){
            $sql = 'UPDATE users_attendance SET inward_datetime = (SELECT created FROM user_attendance_history WHERE id='.$request->att_id.'),inward_success_id='.$request->att_id.' WHERE id='.$id;
            DB::insert($sql);
            return redirect()->route('employee.attendance',[$emp_id])->with('success', 'Employee attendance updated successfully');  
        }
        if($request->out_entry == 'Mark as OUT'){
            $sql = 'UPDATE users_attendance SET outword_datetime = (SELECT created FROM user_attendance_history WHERE id='.$request->att_id.'),outword_success_id='.$request->att_id.' WHERE id='.$id;
            DB::insert($sql);
            return redirect()->route('employee.attendance',[$emp_id])->with('success', 'Employee attendance updated successfully');  
        }
        
        $aData['attendance_id'] = $id;
        $aData['attendance_date'] = $request->date;
        return view('employee.attendance_log', $aData); 
    }

    public function clear_attendance_search($id){
 
        session::forget('start_date');
        session::forget('end_date');
        return redirect()->route('employee.attendance', $id);
     }
    public function attendance(Request $request, $id)
    {
        $aData = array();
        $aData['search'] = '';
        if($request->form_type=='search_frm'){
          
            session(['start_date' => $request->start_date]);
            session(['end_date' => $request->end_date]);
        }

        $aData['start_date'] = session::has('start_date') ? session::get('start_date').' 00:00:00': date('d-m-Y 00:00:00', strtotime('-1 month'));
        $aData['end_date'] = session::has('end_date') ? session::get('end_date') .' 23:59:59' : date('d-m-Y 23:59:59');
       
        if(($aData['start_date']) || ($aData['end_date'])){
            $aData['search'] = 1;
        }

        $aData['aAttendance'] = DB::table('users_attendance as ua')
                                        ->where('ua.user_id', $id)
                                        ->whereBetween('ua.attendance_date',[strtotime($aData['start_date']), strtotime($aData['end_date'])])
                                        ->orderBy('id', 'DESC')->paginate(config('custom.page_limit'));   
        
        $aData['Emp_data']= DB::table('biometric_users')
                                ->where('id', $id)
                                ->select(
                                    'id',
                                    'firstname',
                                    'lastname',
                                    'middlename',
                                    'emp_id'
                                )
                                ->get(); 

        return view('employee.attendance_Details',$aData);
    }

    public function attendance_search(Request $request){

        session(['Pakage_id' => $request->Pakage_id]);
        session(['start_date' => $request->start_date]);
        session(['end_date' => $request->end_date]);
        
        return redirect()->route('employee.list');  
    }

    /* Map */

    public function showroute(Request $request){

        $goole_map_key = config('custom.google_map_key');
        
        $html = '<div class="modal-dialog modal-dialog-centered modal-lg" style="min-width:1400px">
                    <div class="modal-content">
                        <div class="modal-body">
                       <div style="height:700px;" id="map">
                        </div>
                    </div>
                </div>';  
        $html .='

        <script>
                   
                function initMap(){
                    var center = {lat: '.$request->lat.', lng: '.$request->long.'};
                    var gMapoptions = {
                        zoom: 12,
                        center: center,
                        mapTypeId: google.maps.MapTypeId.ROADMAP
                    };	
                
                    var map = new google.maps.Map(document.getElementById("map"),gMapoptions);
                    var marker = new google.maps.Marker({
                        position:center,
                        map: map,
                        });
                    }

            $( document ).ready(function() {
                setTimeout(function(){ initMap()}, 1000);
            });      
                     
        </script>
        ';

        echo $html;
    }
    public function change_status(Request $request){
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        if($request->id){
            $Emplyee = Employee::find($request->id);
            $Emplyee->is_active= $request->status;
            $Emplyee->save();

            $sql = 'UPDATE biometric_users SET is_active='.$request->status.' WHERE id='.$request->id;
            if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
                $session_div = Session::get('Pakages'); 
                if(!empty($session_div)){
                    $session_div = implode(',',$session_div);
                    $sql .= ' AND Pakage_id IN ('.$session_div.')';
                }else {
                    $sql .= '1=2';    
                    echo 0; die;
                }
            }       
            DB::select( DB::raw($sql ), array());
            echo 1;
        }else{
            echo 0;
        }
    }

    // public function contact_validation(Request $request){
    //         if($request->id == "0"){
    //             $this->validate($request,[
    //                 'contact_no' => 'unique:biometric_users'
    //             ]);
    //         }else{
    //             $this->validate($request,[
    //                 'contact_no' => 'unique:biometric_users,contact_no,'. $request->id .'id',
    //             ]);
    //         }
    //         return response("1");
    // }

    public function contact_validation(Request $request){
        $contact_no =  $request->contact_no;
        $edit_id    =  $request->id;
        //echo $edit_id; die;

        $sSQL = 'select count(id) as totalCount from biometric_users where 1=1 ';
        $sSQL .= ' and contact_no = '.$contact_no;

        if($edit_id != 0){
           $sSQL .= ' and id != '.$edit_id;
        }

		$rVRes = DB::select( DB::raw($sSQL), array());
		if(!empty($rVRes)){
			$Result = collect($rVRes)->first();
			$count = $Result->totalCount;
		}

        if($count == 1)
        {
           echo '1'; die;
        }
        else
        {
           echo '0'; die;
        }

    }
    
    //-------------- employee import ---------------------
    public function import_employee(Request $request){

       //dd($request);
        Excel::import(new EmployeedetailsImport($request->Pakage_id), request()->file('emp_file'),\Maatwebsite\Excel\Excel::XLSX);
        return redirect()->route('employee.list', $request->route_id)->with('success', 'Employee updated Successfully');	
    }
}

