<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\VehicleRoute;
use App\Models\VehicleModel;
use App\Models\WardModel;
use App\Libraries\Mysecurity;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\DB;


class VehicleRouteController extends Controller
{
    public function __construct()
    {
        $module_link = request()->segments(0);
        if(isset($module_link[0])){
            $module_link = $module_link[0];
            $this->mysecurity = new Mysecurity();
            $this->module_id = $this->mysecurity->get_module_id_by_name($module_link);
            $this->middleware(function ($request, $next){
                $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,1);
        
                return $next($request);
            });
        }
    }

    public function clear_search(){
        session::forget('Pakage_id');
        session::forget('ward_id');
        session::forget('vehicle_number');
        session::forget('route_id');
        session::forget('day');
        return redirect()->route('vehicle_routes.list');
    }
    
    public function list(Request $request)
    {
      
        $aData = array();
        $aData['search'] = '';
    
        $aData['vehicle_number'] = session::has('vehicle_number') ? session::get('vehicle_number') : '';
        $aData['Pakage_id']      = session::has('Pakage_id') ? session::get('Pakage_id') : '';
        $aData['ward_id']        = session::has('ward_id') ? session::get('ward_id') : '';
        $aData['route_id']       = session::has('route_id') ? session::get('route_id') : '';
        $aData['day']            = session::has('day') ? session::get('day') : '';

        if(!empty($aData['Pakage_id']) || !empty($aData['ward_id']) || !empty($aData['vehicle_number']) || !empty($aData['route_id']) || !empty($aData['day'])){
            $aData['search'] = 1;
        }
    
        $aData['adays'] = DB::table('master_days')->get(); 
        $aData['aVehiclesRoutes'] = VehicleRoute::get($aData)->orderBy('id', 'DESC')->paginate(config('custom.page_limit'));
        $aData['aVehicleNumber'] = VehicleModel::get_vehicle()->orderBy('vehicle_number', 'ASC')->get();
        $aData['Pakages'] = VehicleModel::get_Pakages()->orderBy('Pakage_name', 'ASC')->get();
        $aData['wards'] = VehicleModel::get_wards()->orderBy('ward_no', 'ASC')->get();
        $aData['aRoutes'] = VehicleRoute::get_route()->orderBy('route_name', 'ASC')->get();

         //echo '<pre>';print_r($aData['aVehiclesRoutes']);die();  
       
        return view('vehicle_routes.list', $aData);
        
    }
    
    public function search(Request $request){
        session(['Pakage_id' => $request->Pakage_id]);
        session(['ward_id' => $request->ward_id]);
        session(['vehicle_number' => $request->vehicle_number]);
        session(['route_id' => $request->route_id]); 
        session(['day' => $request->day]); 
    
        return redirect()->route('vehicle_routes.list');
    }
    

    public function add_edit(Request $request, $id=null){

        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);

        $aData = array(); 

        if ($id == '') {  // add vehicle route
            $vehicles_route = new VehicleRoute;
             

        } else {  // edit vehicle route
            $vehicles_route = VehicleRoute::find($id);
        }

        $aData['aRoutes'] = VehicleRoute::get_route()->orderBy('route_name', 'ASC')->get();

        $aData['aVehicleNumber'] = VehicleModel::get_vehicle()->orderBy('vehicle_number', 'ASC')->get();
        $aData['adays'] = DB::table('master_days')->get(); 

        $aData['aTypes'] = VehicleModel::get_vehicle_sub_type();  
        
        if($request->isMethod('post')){
            $current_date = date('Ymd');
              
                //------ if daily route details table from route id is 0 then this condition update for route_id in using vehicle route table  (Add/Edit)            
                $sql = 'select vehicle_number from daily_route_details where vehicle_number = '.$request->vehicle_number.' and route_id = 0 and current_day = '.$current_date;
                $rResult = DB::select( DB::raw($sql), array());
             
                if(!empty($rResult) && !empty($current_date)){
                    $vehicle_id  = $rResult[0]->vehicle_number;
                    $route_id    = $request->route_id;
                    $aData['vehicleRoute'] = VehicleRoute::update_route_id($route_id,$vehicle_id);
                }

            $this->validate($request,[
                'route_id' => 'required',
                'vehicle_number' => 'required',
                'type' => 'required',
                'day' =>'required'
            ],[
                'vehicle_number.unique' =>'This vehicle number is already exist for selected route',
            ]);
        
            $aData = $request->all();
            $vehicles_route->route_id = $aData['route_id'];
            $vehicles_route->type = $aData['type'];
            $vehicles_route->vehicle_number = $aData['vehicle_number'];

            if ($id == null) {
                // If it iis a new entry then create separate entries for each selected day
                $selectedDays = $aData['day'];
                foreach ($selectedDays as $selectedDay) {
                   
                // If $sql,$sql2  check not same date and not already exits route another vehicle by monu 29-11-2023 
                    $sql = "SELECT count(id) as count FROM vehicle_routes vr WHERE route_id = '".$vehicles_route->route_id."' AND vehicle_number = '".$vehicles_route->vehicle_number."' AND day = '".$selectedDay."' ";
                    $check_res = DB::select($sql);

                    $sql2 = "SELECT count(id) as num_count FROM vehicle_routes vr WHERE route_id = '".$vehicles_route->route_id."' AND vehicle_number != '".$vehicles_route->vehicle_number."' AND day = '".$selectedDay."' ";
                    $check_res2 = DB::select($sql2);

                    //echo '<pre>';print_r($check_res2);die();

                    if($check_res[0]->count < 1 && $check_res2[0]->num_count < 1){

                        $newVehicleRoute = new VehicleRoute();
                        $newVehicleRoute->route_id = $vehicles_route->route_id;
                        $newVehicleRoute->type = $vehicles_route->type;
                        $newVehicleRoute->vehicle_number = $vehicles_route->vehicle_number;
                        $newVehicleRoute->day = $selectedDay; // Store each day as a separate entry
                        $newVehicleRoute->save();
                      
                    }
                }
                
               // dd( $newVehicleRoute);
            } else {
                
                 // If $sql,$sql2  check not same date and not already exits route another vehicle by monu 29-11-2023 
                $sql = "SELECT count(id) as count FROM vehicle_routes vr WHERE route_id = '".$aData['route_id']."' AND vehicle_number = '".$aData['vehicle_number']."' AND day = '".$aData['day']."' AND id != '".$id."' ";
                $check_res = DB::select($sql);

                $sql2 = "SELECT count(vehicle_number) as num_count FROM vehicle_routes vr WHERE route_id = '".$aData['route_id']."' AND vehicle_number != '".$aData['vehicle_number']."' AND day = '".$aData['day']."' AND id != '".$id."' ";
                $check_res2 = DB::select($sql2);

                if($check_res[0]->count < 1 && $check_res2[0]->num_count < 1){
                    $existingRoute = VehicleRoute::find($id);
                    if ($existingRoute) {
                            // Update the fields
                        $existingRoute->route_id = $aData['route_id'];
                        $existingRoute->type = $aData['type'];
                        $existingRoute->vehicle_number = $aData['vehicle_number'];                    
                        $existingRoute->day = $aData['day'];// Update the day here and that assuming day is a single day value here
                        $existingRoute->save();
                    }
                }
           // dd( $existingRoute);
            // $vehicles_route->day = json_encode($aData['day']);//for array format
            // $vehicles_route->save();    
        }    
            return redirect()->route('vehicle_routes.list')->with('success', 'Vehicle Route Saved Successfully'); 
        
        }else{

            $aData['aVehiclesRoute'] =$vehicles_route;  
            return view('vehicle_routes.add_edit', $aData); 
        }

    }

    public function delete(Request $request, $id)
    {
        $this->mysecurity->check_access(Session::get('logged_in')['id'],$this->module_id,2);
        VehicleRoute::find($id)->delete();
        return back()->with('success', 'Vehicle Route deleted successfully');
    }

    public function get_wards(Request $request)
    {
        $aWards = WardModel::where('Pakage_id',$request->Pakage_id)->get();
        $assigWards = WardModel::where('id',$request->ward_id)
                                ->where('Pakage_id',$request->Pakage_id)
                                ->select('ward_no')->first();
        $userWards = array();
        if(isset($assigWards->ward_no)){
            $userWards = explode(',',$assigWards->ward_no);
        }
        
        $html = '';
        $html .= '<option value="0">--- Select Wards ---</option>';  
        foreach($aWards as $val){
            $selected = '';           
            if(in_array($val->id,$userWards)){
                $selected = 'selected';
            } 
          
            $html .= '<option '.$selected.' value="'.$val->id.'">'.$val->ward_name.'</option>';
        }
        return $html;
    }


}
