<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Session;

class RouteModel extends Model
{
    use HasFactory;

    protected $table = 'route_master';

    protected $fillable = [
        'route_name',
        'route_file',
        'Pakage_id',
        'ward_id_old',
        'ward_id',
        'stopage_date'
    ];
    
    public $timestamps = false;

    public static function get($aSearch){
        //$aSearch = '';
        $aData = array();

        $query = DB::table('route_master as rm')
                    ->leftjoin('route_checkpoints as rc','rm.id','rc.route_id')
                    ->leftjoin('vehicle_master as vm','rm.vehicle_id','vm.id')
                    ->leftjoin('vehicle_subtypes as vs','rm.sub_type_id','vs.id')
                    ->leftjoin('vehicle_types as vt','vs.vehicle_type_id','vt.id')
                    ->select('rm.*','vm.vehicle_number',DB::raw('concat(vs.name, "(", vt.type, ")") as type_name'), DB::raw("count(rc.id) as checkpoint"));

                    if(!empty($aSearch['Pakage_id']) && $aSearch['Pakage_id'] > 0){
                        $query->where('rm.Pakage_id',$aSearch['Pakage_id']);
                    }
                    if(!empty($aSearch['ward_id']) && $aSearch['ward_id'] > 0){
                        $query->where('rm.ward_id',$aSearch['ward_id']);
                    }
                    if(!empty($aSearch['route_id']) && $aSearch['route_id'] > 0){
                        $query->where('rm.id',$aSearch['route_id']);
                    }

                    if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
                        $session_div = Session::get('Pakages'); 
                        if(!empty($session_div)){
                            $query->whereIn('rm.Pakage_id',$session_div);    
                        }else {
                            $query->whereRaw('1=2');  
                        }
                    }

        $aData = $query->groupBy('rm.id')->orderBy('rm.id', 'ASC')->paginate(30);

           foreach($aData as $res){
           
             $wardName =  DB::table('admin_wards')->where('id',$res->ward_id)->value('ward_name');
             $PakageName =  DB::table('admin_pakages')->where('id',$res->Pakage_id)->value('Pakage_name');
             $res->ward_name = $wardName ;
             $res->Pakage_name = $PakageName ;
           }         
         
        return $aData;
    }

    public static function get_default_stopage_date($iRouteId){

        $query = DB::table('route_master')
        ->select('stopage_date');
        if(!empty($iRouteId)){
            $query->where('id',$iRouteId);
        }
        $result = $query->get()->first();
        return $result->stopage_date;
    }

    public static function get_stoppage_points($iRouteId,$date){
    
        $aReturn = array();
		$vehicle_number = '';
        $query = DB::table('daily_route_details')
        ->select('vehicle_number');
        if(!empty($iRouteId)){
            $query->where('route_id',$iRouteId);
        }

        $result = $query->get();
        $row = $result->first();
        if(!empty($row)){
           $vehicle_number = $row->vehicle_number;
        }
       
        $emp_latest_folder = base_path('latest');
        
        if(!empty($vehicle_number)){
            $actual_route = array();
            $filepath = $emp_latest_folder.'/'.date('Ymd',strtotime($date)).'/'.$vehicle_number.'.json';
           
            if(file_exists($filepath)){
              
                $handle = fopen($filepath, "r");
                $actual_route = fgets($handle);
                fclose($handle);
                $actual_route = trim($actual_route);
                $firstchar = substr($actual_route,0,1);
                if($firstchar != '['){
                    $actual_route = '['.$actual_route.']';
                }
                $actual_route = json_decode($actual_route); 
                usort($actual_route, function($a, $b)
                {
                    return strcmp($a->record_timestamp, $b->record_timestamp);
                });

                if(!empty($actual_route)){
                    foreach($actual_route as $aRoute){
                        $aRoute->lat = (double)$aRoute->latitude;
                        $aRoute->lng = (double)$aRoute->longitude;
                    }
                }
                $aReturn['actual_route'] = $actual_route;
            }
        }
      return $aReturn;   
    }

    public static function get_route($iRouteId){
		$aData = array();
		
        $query = DB::table('route_details as rd')
        ->select('rd.*');
        if(!empty($iRouteId)){
            $query->where('route_id',$iRouteId);
        }
		$result = $query->get();
        //dd($result);
		foreach($result as $aRes){
			$aRes->lat = (double)$aRes->latitude;
			$aRes->lng = (double)$aRes->longitude;
			$aData[] = $aRes;
		}
	   return $aData;
       
	}

    public static function get_checkpoints($iRouteId,$date = ''){
		$aResult = array();
		if(!empty($date)){
			$today = $date;
		}else{
			$today = date('Ymd',strtotime('now'));
		}
	
		$sSQL = 'SELECT rc.*,(SELECT route_name FROM route_master WHERE id = rc.route_id) as route_name,(SELECT TIME_FORMAT(actual_reach_time, "%h:%i %p") as actual_reach_time FROM checkpoint_arrival 
        WHERE route_id = rc.route_id AND checkpoint = rc.checkpoint_number AND date = '.$today.' LIMIT 0,1) as actual_reach_time FROM route_checkpoints rc WHERE rc.route_id = '.$iRouteId.' ORDER BY id';
        
        $rResult = DB::select( DB::raw($sSQL));
        ///$res = collect($AnalyticsData)->count();
        //dd($res);
        foreach($rResult as $aRes){
            $aRes->lat = (double)$aRes->latitude;
			$aRes->lng = (double)$aRes->longitude;
			$aResult[] = $aRes;
		}
		return $aResult;
	}

    public static function get_route_polygon($iRouteId){
		$sSQL = 'SELECT rp.*,rm.route_name FROM route_polygon rp LEFT JOIN route_master rm ON rp.route_id = rm.id  WHERE rp.route_id = '.$iRouteId;
        $rResult = DB::select( DB::raw($sSQL));
		return $rResult;	
	}

    public static function get_route_polygon_checkpoint($iRouteId){
        $sSQL = 'SELECT rpc.* FROM route_polygon_checkpoint rpc WHERE rpc.route_id = '.$iRouteId;
        $rResult = DB::select( DB::raw($sSQL));
        return $rResult;    
    }


    public static function get_vehicle_master(){
       $sSQL = 'SELECT vm.* FROM vehicle_master vm WHERE vm.is_active = 1';
        $rResult = DB::select( DB::raw($sSQL));
        return $rResult;    
    }
    public static function insert_data_pocket($insert_data){
        
        $query = 'INSERT INTO route_polygon (route_id,points,number_of_passes,pocket_name,total_distance) VALUES (:route_id,:points,:number_of_pass_id,:pocket_name,:total_distance)';

        $Result = DB::insert($query, $insert_data);

       return $Result;
    }

    public static function insert_data_marker($insert_data){
        
        $query = 'INSERT INTO route_polygon_checkpoint (route_id,route_polygon_id,latitude,longitude) VALUES (:route_id,:route_polygon_id,:latitude,:longitude)';

        $Result = DB::insert($query, $insert_data);

       return $Result;
    }

    public static function deleted_data_pocket($delete_data){
        
        $query = 'DELETE FROM route_polygon WHERE id=:id';

        $Result = DB::delete($query, $delete_data);

        $query2 = 'DELETE FROM route_polygon_checkpoint WHERE route_polygon_id=:id';

        $Result2 = DB::delete($query2, $delete_data);

       return $Result;
    }

    public static function update_data_pocket($update_data){
        
        $query = 'UPDATE route_polygon
        SET pocket_name    = :pocket_name,
        number_of_passes   = :number_of_pass,
        total_distance     = :total_distance
        WHERE id           = :id';

        $Result = DB::update($query, $update_data);

        return $Result;
    }

    public static function edit_update_data_pocket($update_data){
        
        $query = 'UPDATE route_polygon
        SET points         = :points
        WHERE id           = :id';

        $Result = DB::update($query, $update_data);

        return $Result;
    }

    public static function deleted_data_marker($delete_data){
        
        $query = 'DELETE FROM route_polygon_checkpoint WHERE id=:id';

        $Result = DB::delete($query, $delete_data);

       return $Result;
    }

    public static function deleted_data_all_checkpoint($delete_data){
        
        $query = 'DELETE FROM route_polygon_checkpoint WHERE route_polygon_id=:route_polygon_id';

        $Result = DB::delete($query, $delete_data);

       return $Result;
    }

    public static function get_checkpoint($iRouteId){
		$aResult = array();
		$sSQL = 'SELECT * FROM route_checkpoints WHERE route_id = '.$iRouteId.' ORDER BY id';
		$rResult = DB::select( DB::raw($sSQL));
		foreach($rResult as $aRes){
			$checkpoint = $aRes->checkpoint_number;
			$aResult[] = $checkpoint;
		}
		return $aResult;
	}

    public static function get_checkpoint_details($chkpt_id){
		
        $query = DB::table('route_checkpoints as rc')
        ->select('rc.*');
        if(!empty($chkpt_id)){
            $query->where('id',$chkpt_id);
        }
        $result = $query->get()->first();
        return $result;
	}
    public static function edit_update_vehicle_route($update_data){

        $query = 'UPDATE route_master
        SET vehicle_id       = :vehicle_id,
        sub_type_id          = :sub_type_id,
        start_date           = :start_date,
        vehicle_assign_day   = :vehicle_assign_day,
        repeat_how_many_days = :repeat_days_num
        WHERE id             = :id';

        $Result = DB::update($query, $update_data);

        return $Result;
    }
    public static function remove_vehicle_route_calender($data_arr){

        $sSQL = 'SELECT rm.* FROM route_master rm WHERE rm.id = '.$data_arr['id'];
        
        $rResult = DB::select($sSQL);

        $current_day = strtotime('now');

        if($rResult){
             
            $qry = 'INSERT INTO route_master_log (route_id,route_name,route_file,Pakage_id,ward_id_old,ward_id,stopage_date,vehicle_id,sub_type_id,start_date,current_day,repeat_how_many_days) VALUES ("'.$rResult[0]->id.'","'.$rResult[0]->route_name.'","'.$rResult[0]->route_file.'","'.$rResult[0]->Pakage_id.'","'.$rResult[0]->ward_id_old.'","'.$rResult[0]->ward_id.'","'.$rResult[0]->stopage_date.'","'.$rResult[0]->vehicle_id.'","'.$rResult[0]->sub_type_id.'","'.$rResult[0]->start_date.'","'.$current_day.'","'.$rResult[0]->repeat_how_many_days.'")';

                //echo $qry;die;

            $r_result = DB::insert($qry);
        }

        $query = 'UPDATE route_master
        SET vehicle_id       = :vehicle_id,
        sub_type_id          = :sub_type_id,
        start_date           = :start_date,
        repeat_how_many_days = :repeat_days_num
        WHERE id             = :id';

        $Result = DB::update($query, $data_arr);

        return $Result;
    }

    public static function get_latest_locations($aSearch){
        //  echo '<pre>';print_r($aSearch);
        $aData = array();

        if(date('H',strtotime('now')) <= 06 ){
            $today = date('Ymd',strtotime('-1 day'));
        }else{
            $today = date('Ymd',strtotime('now'));
        }
        
        // $sSQL = 'SELECT drd.*,rm.Pakage_id,rm.route_name as route_name,(SELECT Pakage_name from admin_pakages WHERE id = rm.Pakage_id) as Pakage_name,(SELECT vehicle_number from vehicle_master WHERE id = drd.vehicle_number) as vehicle_number_to_show,(SELECT description from vehicle_master WHERE id = drd.vehicle_number) as veh_description FROM daily_route_details drd LEFT JOIN route_master rm ON drd.route_id = rm.id LEFT JOIN vehicle_routes vr ON vr.route_id = drd.route_id WHERE drd.current_day = '.$today;


        $sSQL = 'SELECT drd.*,vm.Pakage_id,(select route_name from route_master WHERE id=max(drd.route_id)) as route_name,(SELECT Pakage_name from admin_pakages WHERE id = vm.Pakage_id) as Pakage_name,(SELECT vehicle_number from vehicle_master WHERE id = drd.vehicle_number) as vehicle_number_to_show,(SELECT description from vehicle_master WHERE id = drd.vehicle_number) as veh_description FROM daily_route_details drd LEFT JOIN vehicle_master vm ON drd.vehicle_number = vm.id WHERE drd.current_day = '.$today;

        // echo $sSQL;

        if($aSearch['Pakage_id'] > 0){
            $sSQL .= ' AND vm.Pakage_id = '.$aSearch['Pakage_id'];
        }	
        
        // if($aSearch['ward_id'] > 0){
        //     $sSQL .= ' AND rm.ward_id = '.$aSearch['ward_id'];
        // }	

        if($aSearch['vehicle_id'] > 0){
            $sSQL .= ' AND drd.vehicle_number = '.$aSearch['vehicle_id'];
        }
        
        if($aSearch['vehicle_type_id'] > 0){
            $sSQL .= ' AND drd.type = '.$aSearch['vehicle_type_id'];
        }
        
        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $sSQL .= ' AND vm.Pakage_id IN ('.$str_div.')';
            }else {
                $sSQL .= ' AND 1=2';
            }
        }
        $sSQL .= ' group by vehicle_number';
        //echo $sSQL;die;
        $rResult = DB::select( DB::raw($sSQL), array());

         // echo '<pre>';print_r($rResult);die();
        $div_colors = config('custom.divColors');
        $update_time_limit = config('custom.update_time_limit');

        $avial_divs = array();
        foreach($rResult as $key=>$aRes){
        
            $latest_file = base_path('public/latest/'.$aRes->device_number.'.json');

            //echo '<pre>';print_r($latest_file);die();
            if(file_exists($latest_file)){
                $handle = fopen($latest_file, "r");
                $vehicle_entries = fgets($handle);
                fclose($handle);
                $vehicle_entries = trim($vehicle_entries);	
                
                if(!empty($vehicle_entries)){
                    $vehicle_entries = json_decode($vehicle_entries,true);
                    if(!empty($vehicle_entries)){
                        $rResult[$key]->current_lat   = $vehicle_entries['Latitude'];                    
                        $rResult[$key]->current_long = $vehicle_entries['Longitude'];
                        $rResult[$key]->speed = $vehicle_entries['Speed'];
                        $rResult[$key]->latest_time = strtotime($vehicle_entries['Datetime']);
                        $rResult[$key]->latest_datetime = $vehicle_entries['Datetime'];
                    }
                }
            }

            $aRes->current_lat = (isset($aRes->current_lat)) ? $aRes->current_lat : $aRes->latest_latitude;
			$aRes->current_long = (isset($aRes->current_long)) ? $aRes->current_long : $aRes->latest_longitude;
			$aRes->speed = (isset($aRes->speed)) ? $aRes->speed : $aRes->latest_speed;

            // echo '<pre>';print_r($aRes);die();

            if(!empty($aRes->Pakage_id))
                $aRes->div_color = $div_colors[$aRes->Pakage_id];
            else
           // $aRes->div_color = '#000000';
            $aRes->div_color = '#ababab';
                //dd($aRes->div_color);
            if(!in_array($aRes->Pakage_id,$avial_divs)){
				$aData['markers']['marker_Pakages'][] = $aRes;
				$avial_divs[] = $aRes->Pakage_id;
			}

            if($aRes->latest_time > $update_time_limit){
				$aData['markers']['above_speed_markers'][] = $aRes;
			}else{
				$aData['markers']['below_speed_markers'][] = $aRes;			
			}
        }
        
       // echo '<pre>';print_r($aData);die();
        return $aData;
    }

    public static function get_prabhag_boundray($aSearch){
       //print_r($wardID); die;
		$aData = array();
		$sSQL = 'SELECT latitude,longitude,ward_id FROM prabhag_boundary WHERE 1=1';
        $sSQL .= ' AND ward_id = '.$aSearch['ward_id'];
    
        $rResult = DB::select( DB::raw($sSQL));
        //dd($rResult);
		foreach($rResult as $aRes){
            $aRes->lat = (double)$aRes->latitude;
			$aRes->lng = (double)$aRes->longitude;
            $aData[] = $aRes;
		}
       // print_r($aData); die;
		return $aData;	
	}

    public static function get_present_count($aSearch){
        $aData = array();
		
        if(date('H',strtotime('now')) <= 06 ){
            $today = date('Ymd',strtotime('-1 day'));
        }else{
            $today = date('Ymd',strtotime('now'));
        }
	
        $query = DB::table('daily_route_details as drd')
        ->leftjoin('vehicle_master as vm','drd.vehicle_number','vm.id')
        ->select('drd.id')
        ->groupBy('drd.vehicle_number');
        if(!empty($today)){
            $query->where('drd.current_day',$today);
        }
        if($aSearch['Pakage_id'] > 0){
            $query->where('vm.Pakage_id',$aSearch['Pakage_id']);
        }
        if($aSearch['vehicle_id'] > 0){
            $query->where('drd.vehicle_number',$aSearch['vehicle_id']);
        }
        if($aSearch['vehicle_type_id'] > 0){
            $query->where('drd.type',$aSearch['vehicle_type_id']);
        }
        if($aSearch['route_id'] > 0){
            $query->where('drd.route_id',$aSearch['route_id']);
        }

        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $query->whereIn('vm.Pakage_id',$session_div);    
            }else {
                $query->whereRaw('1=2'); 
            }
        }

        $result = $query->get();
        $count = $result->count();
        //echo '<pre>';print_r($result);die();
		return $count;
    }

    public static function get_all_routes($aSearch){
      // dd($aSearch);
        $aRoutes = array();
        //$aSearch = '';
		$defined_sSQL = 'SELECT rd.*,rm.Pakage_id,rm.route_name as route_name,vr.id as vehicleId,vr.vehicle_number FROM route_details rd LEFT JOIN route_master rm ON rd.route_id = rm.id LEFT JOIN vehicle_routes vr ON vr.route_id = rm.id WHERE 1=1';

        if($aSearch['Pakage_id'] > 0){
            $defined_sSQL .= ' AND rm.Pakage_id = '.$aSearch['Pakage_id'];
        }	
        
        if($aSearch['ward_id'] > 0){
            $defined_sSQL .= ' AND rm.ward_id = '.$aSearch['ward_id'];
        }	

        if(Session::has('logged_in.superuser') && Session::get('logged_in.superuser') !=1){
            $session_div = Session::get('Pakages'); 
            if(!empty($session_div)){
                $str_div = implode(',',$session_div);
                $defined_sSQL .= ' AND rm.Pakage_id IN ('.$str_div.')';
            }else {
                $defined_sSQL .= ' AND 1=2';
            }
        }

        $rRoute_details = DB::select( DB::raw($defined_sSQL), array());
		//dd($rRoute_details);
		$aColors = array();
		$iColOffset = 0;
        $divColors = config('custom.divColors');
		foreach($rRoute_details as $aRow){
			$aRow->lat = (double)$aRow->latitude;
			$aRow->lng = (double)$aRow->longitude;
			$aRoutes[$aRow->route_id]['route_name'] = $aRow->route_name;
			$aRoutes[$aRow->route_id]['vahicle_number'] = $aRow->vehicle_number;
			$aRoutes[$aRow->route_id]['route_id'] = $aRow->route_id;
			$aRoutes[$aRow->route_id]['vehicleId'] = $aRow->vehicleId;
			$aRoutes[$aRow->route_id]['Pakage_id'] = $aRow->Pakage_id;
			$aRoutes[$aRow->route_id]['divColor'] = (in_array($aRow->Pakage_id,$divColors)) ? $divColors[$aRow->Pakage_id] : 'white';
			if(empty($aRoutes[$aRow->route_id]['color'])){
				$color = RouteModel::random_color();
				while(!in_array($color,$aColors)){
					$aRoutes[$aRow->route_id]['color'] = '#'.$color;
					$aColors[] = $color;
				}
			}
			$aRoutes[$aRow->route_id]['defined_route'][] = $aRow;
		}
        //dd($aRoutes);
		return $aRoutes;
    }

    public static function random_color_part() {
		return str_pad( dechex( mt_rand( 0, 255 ) ), 2, '0', STR_PAD_LEFT);
	}

    public static function random_color() {
		return RouteModel::random_color_part() . RouteModel::random_color_part() . RouteModel::random_color_part();
	}
    public static function get_to_fetch_route($aSeach=''){

        $sql = 'SELECT rm.* FROM route_master rm WHERE 1=1';
        
        if(!empty($aSeach['packege_id'])){

        $sql .= ' AND Pakage_id = '.$aSeach['packege_id'];
        
        }

       $Result = DB::select($sql); 

       return $Result;
    }
    
}

?>